#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: on_wait_statistics_gathering.sh,v $
#	$Revision: 1.3 $
#	$Date: 2007/12/11 12:58:32 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2008.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script contains the initial statistics gathering functions
#

# This function collects the memory-usage statistics.
#   VSZ - (Virtual memory SIZE) - The amount of memory the process is
#         currently using. This includes the amount in RAM and the
#         amount in swap.
#   RSS - (Resident Set Size) - The portion of a process that exists in
#         physical memory (RAM). The rest of the program exists in swap.
#         If the computer has not used swap, this number will be equal
#         to VSIZE.
# The result is in ${2} file and it is three lines of the form " VSZ RSS "
# where first line is the pair of ps results with the max VSZ, the second
# is the one with the max RSS and the third one is the average over all measurements.
# NOTE: We do not know the number of measurements we will have to do in advance,
#       and we may get overflow if we try to first sum up all the "VSZ RSS" pairs
#       and then divide the result by the total number of measurements.
#       That is why we use recursive formula, which might suffer from round off
#       errors though. For the sequence of measurements:
#           c_{1}, c_{2}, c_{3}, ... c_{n-1}, c_{n}, ...
#       we define the on-the-fly average on each iteration as:
#           A_{1} = c_{1}
#           A_{n} = A_{n-1} * ( n-1 ) / n + c_{n} / n
# @param ${1} the monitored process PID
# @param ${2} the data output file name
# @param ${3} the iteration number, should be equal 1 for the first call
#             for any given test case
#
collectMemotyUsage () {
	#Define sed pattern
	VSZ_RULE="s/[ ]*[0-9.,]*[ ]*$//"
	RSS_RULE="s/^[ ]*[0-9.,]*[ ]*//"
	TMP_OUT_FILE_NAME="${2}.new"
	#Just in case it is here
	rm -f ${TMP_OUT_FILE_NAME}
	
	#Just collect the statistics first, i.e. "  VSZ  RSS  " string
	MEM_USAGE=`eval ps -p ${1} -o vsz,rss | sed -e '/VSZ/d'`
	#echo -n "New MEM_USAGE=${MEM_USAGE}"
	#echo "Old file:"
	#cat ${2}
	
	#The process could be finished by now and then ${MEM_USAGE} would be empty or zero.
	if ! test -z "${MEM_USAGE}"; then
		#Check if the stat file exists
		if ! test -f ${2}; then
			#if not, create it with three identical lines
			echo ${MEM_USAGE} > ${2}
			echo ${MEM_USAGE} >> ${2}
			echo ${MEM_USAGE} >> ${2}
		else
			#Delete the suffix, i.e. "  RSS  "
			NEW_VSZ_USAGE=`eval echo "${MEM_USAGE}" | sed -e "${VSZ_RULE}"`
			#Delete the prefix, i.e. "  VSZ  "
			NEW_RSS_USAGE=`eval echo "${MEM_USAGE}" | sed -e "${RSS_RULE}"`
			
			#Extract the old data from the file, we expect two lines only
			MAX_VSZ_MEM_USAGE=`eval sed -n '1p' ${2}`
			MAX_RSS_MEM_USAGE=`eval sed -n '2p' ${2}`
			AVERAGE_MEM_USAGE=`eval sed -n '3p' ${2}`
			
			#Check the max value of VSZ, if they are equal then choose the one with greater RSS
			OLD_VSZ_MAX_VSZ_USAGE=`eval echo "${MAX_VSZ_MEM_USAGE}" | sed -e "${VSZ_RULE}"`
			OLD_RSS_MAX_VSZ_USAGE=`eval echo "${MAX_VSZ_MEM_USAGE}" | sed -e "${RSS_RULE}"`
			if test \( "${OLD_VSZ_MAX_VSZ_USAGE}" -lt "${NEW_VSZ_USAGE}" \) -o \( \( "${OLD_VSZ_MAX_VSZ_USAGE}" -eq "${NEW_VSZ_USAGE}" \) -a \( "${OLD_RSS_MAX_VSZ_USAGE}" -lt "${NEW_RSS_USAGE}" \) \) ; then
				echo "${MEM_USAGE}" > ${TMP_OUT_FILE_NAME}
			else
				echo "${MAX_VSZ_MEM_USAGE}" > ${TMP_OUT_FILE_NAME}
			fi
			
			#Check the max value of RSS, if they are equal then choose the one with greater VSZ
			OLD_VSZ_MAX_RSS_USAGE=`eval echo "${MAX_RSS_MEM_USAGE}" | sed -e "${VSZ_RULE}"`
			OLD_RSS_MAX_RSS_USAGE=`eval echo "${MAX_RSS_MEM_USAGE}" | sed -e "${RSS_RULE}"`
			if test \( "${OLD_RSS_MAX_RSS_USAGE}" -lt "${NEW_RSS_USAGE}" \) -o \( \( "${OLD_RSS_MAX_RSS_USAGE}" -eq "${NEW_RSS_USAGE}" \) -a \( "${OLD_VSZ_MAX_RSS_USAGE}" -lt "${NEW_VSZ_USAGE}" \) \); then
				echo "${MEM_USAGE}" >> ${TMP_OUT_FILE_NAME}
			else
				echo "${MAX_RSS_MEM_USAGE}" >> ${TMP_OUT_FILE_NAME}
			fi
			
			#Take the previous average values
			#Check the max value of VSZ, if they are equal then choose the one with greater RSS
			OLD_VSZ_AVERAGE_USAGE=`eval echo "${AVERAGE_MEM_USAGE}" | sed -e "${VSZ_RULE}"`
			OLD_RSS_AVERAGE_USAGE=`eval echo "${AVERAGE_MEM_USAGE}" | sed -e "${RSS_RULE}"`
			NEW_VSZ_AVERAGE_USAGE=`eval awk -v old_average="${OLD_VSZ_AVERAGE_USAGE}" -v new_value="${NEW_VSZ_USAGE}" -v iteration="${3}" -f ${PERFORMANCE_AWK_SCRIPTS}/on_the_fly_average.awk`
			NEW_RSS_AVERAGE_USAGE=`eval awk -v old_average="${OLD_RSS_AVERAGE_USAGE}" -v new_value="${NEW_RSS_USAGE}" -v iteration="${3}" -f ${PERFORMANCE_AWK_SCRIPTS}/on_the_fly_average.awk`
			echo "  ${NEW_VSZ_AVERAGE_USAGE}  ${NEW_RSS_AVERAGE_USAGE}" >> ${TMP_OUT_FILE_NAME}
			
			#Substitute the file with the new one
			rm -f ${2}
			mv ${TMP_OUT_FILE_NAME} ${2}
		fi
		#echo "New file:"
		#cat ${2}
	elif ! test -f ${2}; then
			#The process ended up sooner than we started measurements
			echo "   0   0" > ${2}
			echo "   0   0" >> ${2}
			echo "   0   0" >> ${2}
	fi
}

#The time out for the test run in milliseconds, then is used in the scripts for
#printing the dummy results for the timed out tests
export TIME_OUT_MILLISEC=""

#This function allows for terminating the MRMC performance
#test case execution by the timeout.
#It also calls collection of the pick memory usage.
#@param ${1} is the timeout value.
#@param ${2} is the monitored process ID, should be set to -1 if
#            the process has already terminated.
#@param ${3} is the file where the pick VSZ and RSS will be stored
#            The memory statistics is not collected is ${3} is omitted
#@return 0 if no time-out occured, otherwise 1
#
doWait () {
	let "TIME_OUT_MILLISEC = ${1} * 1000"
	let "NUMBER_OF_ITER = ${1} * 10"	#Right now it is set for SLEEPING_TIME=0.1 second
	SLEEPING_TIME=0.1			#This is how much in seconds we should sleep between iterations.
	let "kill_indx = ${NUMBER_OF_ITER} + 1"
	
	echo -n "0"
	BSP="\b"
	#Test for the case when MRMC has already terminated
	for (( i=1 ; i <= ${NUMBER_OF_ITER} ; i++ )) ; do
		#Sleep for one milli second
		sleep ${SLEEPING_TIME}
		
		#Check whether we have to collect memory statistics
		if ! test -z "${3}"; then
			collectMemotyUsage "${2}" "${3}" "${i}"
		fi
		
		CHECK_MRMC=`eval ps ${2} | grep ${2}`
		if test ! -z "${CHECK_MRMC}"; then
			#Clean the old counter value
			echo -ne "${BSP}${i}"
			#Increase the number of back spaces for the next iteration
			if test \( "${i}" -eq "10" \) -o \( "${i}" -eq "100" \) -o \( "${i}" -eq "1000" \) -o \( "${i}" -eq "10000" \) -o \( "${i}" -eq "100000" \) -o \( "${i}" -eq "1000000" \) -o \( "${i}" -eq "10000000" \) -o \( "${i}" -eq "100000000" \) -o \( "${i}" -eq "1000000000" \) -o \( "${i}" -eq "10000000000" \); then
				BSP="${BSP}\b"
			fi
		else
			let "i = ${NUMBER_OF_ITER} + 2"
		fi
	done
	
	#Test if the monitored process has to be killed because of the time-out
	if test "${i}" -eq "${kill_indx}"; then
		kill -9 ${2}
		echo -ne "${BSP}-"
		return 1
	else
		echo -ne "${BSP}+"
		return 0
	fi
}
